/*
** HDL4SE: VerilogۺϷƽ̨
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** LCOM: ģ
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** * Redistributions of source code must retain the above copyright notice,
**   this list of conditions and the following disclaimer.
** * Redistributions in binary form must reproduce the above copyright notice,
**   this list of conditions and the following disclaimer in the documentation
**   and/or other materials provided with the distribution.
** * The name of the author may be used to endorse or promote products
**   derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
** THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
* hdl4se_wire.c
  ޸ļ¼
	202105221951: rxh, initial version
	202105241522rxh, Detectorӿ
*/
#include "stdlib.h" 
#include "stdio.h"
#include "string.h"
#include "object.h"
#include "dlist.h"
#include "bignumber.h"
#include "hdl4secell.h"

#define MAX_WIRECONNECT 32

/* 
HDL4SE_WIRE
instance parameter: "8", width=8
*/
typedef struct _sHDL4SEWire {
	OBJECT_HEADER
	INTERFACE_DECLARE(IHDL4SEUnit)
	HDL4SEUNIT_VARDECLARE
	INTERFACE_DECLARE(IHDL4SEDetector)
	HDL4SEDETECTOR_VARDECLARE
	DLIST_VARDECLARE
	
	IHDL4SEModule** parent;
	char* name;

	IHDL4SEUnit** wire_in[MAX_WIRECONNECT];
	int wire_in_index[MAX_WIRECONNECT];
	int wire_in_count;

	IBigNumber** out_data;
	int datavalid;
	int width;
}sHDL4SEWire;

OBJECT_FUNCDECLARE(hdl4se_wire, CLSID_HDL4SE_WIRE);
HDL4SEUNIT_FUNCDECLARE(hdl4se_wire, CLSID_HDL4SE_WIRE, sHDL4SEWire);
HDL4SEDETECTOR_FUNCDECLARE(hdl4se_wire, CLSID_HDL4SE_WIRE, sHDL4SEWire);
DLIST_FUNCIMPL(hdl4se_wire, CLSID_HDL4SE_WIRE, sHDL4SEWire);

OBJECT_FUNCIMPL(hdl4se_wire, sHDL4SEWire, CLSID_HDL4SE_WIRE);


QUERYINTERFACE_BEGIN(hdl4se_wire, CLSID_HDL4SE_WIRE)
QUERYINTERFACE_ITEM(IID_HDL4SEUNIT, IHDL4SEUnit, sHDL4SEWire)
QUERYINTERFACE_ITEM(IID_HDL4SEDETECTOR, IHDL4SEDetector, sHDL4SEWire)
QUERYINTERFACE_ITEM(IID_DLIST, IDList, sHDL4SEWire)
QUERYINTERFACE_END

static const char* hdl4se_wireModuleInfo()
{
	return "1.0.0-20210522.1951 HDL4SE wire cell";
}

static int hdl4se_wireCreate(const PARAMITEM* pParams, int paramcount, HOBJECT* pObject)
{
	sHDL4SEWire* pobj;
	int i;
	pobj = (sHDL4SEWire*)malloc(sizeof(sHDL4SEWire));
	if (pobj == NULL)
		return -1;
	*pObject = 0;
	HDL4SEUNIT_VARINIT(pobj, CLSID_HDL4SE_WIRE);
	INTERFACE_INIT(IHDL4SEUnit, pobj, hdl4se_wire, hdl4se_unit);
	INTERFACE_INIT(IHDL4SEDetector, pobj, hdl4se_wire, hdl4se_detector);
	DLIST_VARINIT(pobj, hdl4se_wire);

	pobj->wire_in_count = 0;
	for (i = 0; i < MAX_WIRECONNECT; i++) {
		pobj->wire_in[i] = NULL;
	}
	pobj->out_data = NULL;
	
	pobj->name = NULL;
	pobj->width = 8;
	pobj->datavalid = 0;
	pobj->parent = NULL;

	for (i = 0; i < paramcount; i++) {
		if (pParams[i].name == PARAMID_HDL4SE_UNIT_NAME) {
			if (pobj->name != NULL)
				free(pobj->name);
			pobj->name = strdup(pParams[i].pvalue);
		} 
		else if (pParams[i].name == PARAMID_HDL4SE_UNIT_PARENT) {
			pobj->parent = (IHDL4SEModule **)pParams[i].pvalue;
		}
		else if (pParams[i].name == PARAMID_HDL4SE_UNIT_INSTANCE_PARAMETERS) {
			IBigNumber** temp = bigintegerCreate(32);
			if (temp != NULL) {
				const char* nstr;
				if (0 == objectCall2(temp, AssignStr, (const char*)pParams[i].pvalue, &nstr)) {
					objectCall1(temp, GetInt, &pobj->width);
				}
				objectRelease(temp);
			}
		}
	}

	if (pobj->width <= 0)
		return EIID_INVALIDPARAM;
	pobj->out_data = bigintegerCreate(pobj->width);

	/* ɵĶ */
	OBJECT_RETURN_GEN(hdl4se_wire, pobj, pObject, CLSID_HDL4SE_WIRE);
	return EIID_OK;
}

static void hdl4se_wireDestroy(HOBJECT object)
{
	sHDL4SEWire* pobj;
	int i;
	pobj = (sHDL4SEWire*)objectThis(object);
	if (pobj->name != NULL)
		free(pobj->name);
	for (i = 0; i < pobj->wire_in_count; i++) {
		objectRelease(pobj->wire_in[i]);
	}
	objectRelease(pobj->out_data);
	memset(pobj, 0, sizeof(sHDL4SEWire));
	free(pobj);
}

static int hdl4se_wireValid(HOBJECT object)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	return 1;
}

static int hdl4se_wire_hdl4se_unit_Connect(HOBJECT object, int index, HOBJECT from, int fromindex) 
{
	sHDL4SEWire* pobj;
	IHDL4SEUnit** unit = NULL;
	pobj = (sHDL4SEWire*)objectThis(object);
	if (pobj->wire_in_count >= MAX_WIRECONNECT)
		return -1;
	if (0 == objectQueryInterface(from, IID_HDL4SEUNIT, (void**)&unit)) {
		pobj->wire_in[pobj->wire_in_count] = unit;
		pobj->wire_in_index[pobj->wire_in_count] = fromindex;
		pobj->wire_in_count++;
		return 0;
	}
	return -1;
}

static int hdl4se_wire_hdl4se_unit_GetValue(HOBJECT object, int index, int width, IBigNumber ** value)
{
	int i;
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	if (pobj->datavalid == 0) {
		for (i = 0; i < pobj->wire_in_count; i++) {
			objectCall3(pobj->wire_in[i], GetValue, pobj->wire_in_index[i], pobj->width, pobj->out_data);
		}
		objectCall2(pobj->out_data, SetWidth, pobj->width, 1);
		pobj->datavalid = 1;
	}
	objectCall1(value, Assign, pobj->out_data);
	return 0;
}

static int hdl4se_wire_hdl4se_unit_ClkTick(HOBJECT object)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	return 0;
}

static int hdl4se_wire_hdl4se_unit_Setup(HOBJECT object)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	pobj->datavalid = 0;
	return 0;
}

static int hdl4se_wire_hdl4se_detector_GetName(HOBJECT object, const char** pname)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	*pname = pobj->name;
	return 0;
}

static int hdl4se_wire_hdl4se_detector_GetSignalCount(HOBJECT object)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	return 1;
}

static int hdl4se_wire_hdl4se_detector_GetSignalInfo(HOBJECT object, int index, const char** pname, int * width)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	*pname = "out";
	*width = pobj->width;
	return 0;
}

static int hdl4se_wire_hdl4se_detector_GetSignalValue(HOBJECT object, int index, IBigNumber** value)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	objectCall1(value, Assign, pobj->out_data);
	return 0;
}

static int hdl4se_wire_hdl4se_detector_GetUnitCount(HOBJECT object)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	return 0;
}

static int hdl4se_wire_hdl4se_detector_GetUnit(HOBJECT object, int index, HOBJECT* unit)
{
	sHDL4SEWire* pobj;
	pobj = (sHDL4SEWire*)objectThis(object);
	return 0;
}

